<?php

namespace Controllers;

use Exception;
use Traits\TResponse;
use Zeedhi\Framework\Controller\Simple;
use Zeedhi\Framework\DTO;
use Services\Notice as NoticeService;
use Util\InstanceProvider;

class Notice extends Simple
{

    use TResponse;

    /**
     * @var NoticeService
     */
    private $noticeService;

    /**
     * Notice constructor.
     * @param NoticeService
     */
    public function __construct(NoticeService $noticeService)
    {
        $this->noticeService = $noticeService;
    }

    public function findNotice(DTO\Request $request, DTO\Response $response)
    {
        try {
            $data = $this->noticeService->findNotice();
            $this->sendResponse($data, 200);
        } catch (Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function createNotice(DTO\Request $request, DTO\Response $response)
    {
        try {

            $data = $request->getParameters();
            $this->noticeService->createNotice($data);
            $this->sendResponse(['success' => 'Comunicado adicionado com sucesso!'], 200);
        } catch (Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function deleteNotice(DTO\Request $request, DTO\Response $response)
    {
        try {
            $params = $request->getParameters();
            $this->noticeService->deleteNotice($params['id']);
            $this->sendResponse(['success' => 'Comunicado(s) excluído(s) com sucesso!'], 200);
        } catch (Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function updateNotice(DTO\Request $request, DTO\Response $response)
    {
        try {
            $data = $request->getParameters();
            $this->noticeService->updateNotice($data);
            $this->sendResponse(['success' => 'Comunicado(s) atualizado(s) com sucesso!'], 200);
        } catch (Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function findTimeOutnotice(DTO\Request $request, DTO\Response $response)
    {
        try {
            $data = $this->noticeService->findTimeOutnotice();
            $this->sendResponse($data, 200);
        } catch (Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function findSilencersNotices(DTO\Request $request, DTO\Response $response)
    {
        try {
            $data = $this->noticeService->findSilencersNotices();
            $this->sendResponse($data, 200);
        } catch (Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function saveTimeOutNotice(DTO\Request $request)
    {
        try {
            $this->noticeService->saveTimeOutNotice($request->getParameters());
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function deleteTimeOutNotice(DTO\Request $request)
    {
        try {
            $this->noticeService->deleteTimeOutNotice($request->getParameters());
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function readNotification(DTO\Request $request)
    {
        try {
            $nrInfEcommerce = $request->getParameter("NRINFECOMMERCE");
            $this->noticeService->readNotification($nrInfEcommerce);
            $this->sendResponse(['success' => 'Comunicado marcado como lido.'], 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function saveUserNoticeSilence(DTO\Request $request)
    {
        try {
            $data = $request->getParameters();
            $this->noticeService->saveUserNoticeSilence($data);
            $this->sendResponse(['success' => 'Comunicados silenciados.'], 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getSessionNotifications(DTO\Request $request, DTO\Response $response)
    {
        try {
            $data = array(
                'NOTICES' => array(),
                'TIMEOUTNOTICES' => array()
            );
            $filter = array(
                'TOKEN' => $request->getParameter('LOGIN_TOKEN')
            );
            $noticeForOperatorData = InstanceProvider::getMongoDB()->find('LoginNoticeForOperator', $filter);
            if(!empty($noticeForOperatorData)) {
                $data['NOTICES'] = $noticeForOperatorData[0]['NOTICES'];
                $data['TIMEOUTNOTICES'] = $noticeForOperatorData[0]['TIMEOUTNOTICES'];
                InstanceProvider::getMongoDB()->remove('LoginNoticeForOperator', $filter);
            }
            $this->sendResponse($data, 200);
        } catch (Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }
}
