<?php


namespace Repositories;


use Doctrine\ORM\EntityManager;
use Util\InstanceProvider;
use Teknisa\Model\Traits\TQuery;

class Notice
{
    use TQuery;
    /**
     * @var EntityManager
     */
    private $entityManager;

    /**
     * Notice constructor.
     * @param EntityManager $entityManager
     */
    public function __construct(EntityManager $entityManager)
    {
        $this->entityManager = $entityManager;
    }

    public function getNrorg() {
        $environment = InstanceProvider::getLibEnvironment();
        return $environment->getCurrentOrganizationId();
    }

    public function getCdoperador() {
        $environment = InstanceProvider::getLibEnvironment();
        return $environment->getCdOperador();
    }

    public function getNoticeRecords()
    {
        $query = <<<SQL
        SELECT INF.NRINFECOMMERCE,
               INF.DSTITULOINF,
               INF.TXCONTEUDOINF,
               INF.IDNOTIFICACAO,
               TO_CHAR(INF.DTINFORMARIVI, 'DD/MM/YYYY') AS DTINFORMARIVI,
               TO_CHAR(INF.DTEXPIRAINFO, 'DD/MM/YYYY') AS DTEXPIRAINFO,
               TO_CHAR(INF.DTINCLUSAO, 'DD/MM/YYYY') AS DTINCLUSAO,
               TO_CHAR(INF.DTULTATU, 'DD/MM/YYYY') AS DTULTATU,
               INF.IDATIVO,
               INF.CDPUBLICOALVO
          FROM INFORMATIVOECOMMERCE INF
          WHERE INF.CDPUBLICOALVO = 'L'
          AND NRORG = :NRORG
SQL;

        $params = array(
            "NRORG" => $this->getNrorg()
        );

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $query, $params);
    }

    public function insertNotice($data)
    {
            $query = <<<SQL
            INSERT
              INTO INFORMATIVOECOMMERCE
                   (NRINFECOMMERCE,
                   DSTITULOINF,
                   TXCONTEUDOINF,
                   IDNOTIFICACAO,
                   DTINFORMARIVI,
                   DTEXPIRAINFO,
                   DTINCLUSAO,
                   DTULTATU,
                   IDATIVO,
                   CDPUBLICOALVO,
                   NRORG,
                   NRORGINCLUSAO,
                   CDOPERINCLUSAO)
            VALUES (:ID,
                   :TITULO,
                   :CONTEUDO,
                   :TIPO,
                   TRUNC(CAST(:DATAINICIO AS DATE)),
                   TRUNC(CAST(:DATATERMINO AS DATE)),
                   TRUNC(CAST(SYSDATE AS DATE)),
                   TRUNC(CAST(SYSDATE AS DATE)),
                   :ATIVO,
                   :CDPUBLICOALVO,
                   :NRORG,
                   :NRORGINCLUSAO,
                   :CDOPERINCLUSAO)
SQL;
            $params = array(
                'ID' => $data['id'],
                'TITULO' => $data['title'],
                'CONTEUDO' => $data['contentText'],
                'TIPO' => $data['type'],
                'DATAINICIO' => $data['dateBegin'],
                'DATATERMINO' => $data['dateEnd'],
                'ATIVO' => $data['active'],
                'CDPUBLICOALVO' => 'L',
                'NRORG'         => $this->getNrorg(),
                'NRORGINCLUSAO' => $this->getNrorg(),
                'CDOPERINCLUSAO' => $this->getCdoperador()
            );

        $entityManager = InstanceProvider::getLibConnectionCustom()->updateVpdConnection($this->getNrorg());
        $entityManager = $entityManager ? $entityManager : InstanceProvider::getEntityManager();
        return $entityManager->getConnection()->executeQuery($query, $params);   
    }

    public function deleteNotice($id)
    {
        $query = <<<SQL
        DELETE
            FROM INFORMATIVOECOMMERCE
            WHERE NRINFECOMMERCE = :NRINFECOMMERCE
            AND NRORG = :NRORG
SQL;
        $params = array (
            'NRORG' => $this->getNrorg(),
            'NRINFECOMMERCE'    => $id,
        );
        $entityManager = InstanceProvider::getLibConnectionCustom()->updateVpdConnection($this->getNrorg());
        $entityManager = $entityManager ? $entityManager : InstanceProvider::getEntityManager();
        return $entityManager->getConnection()->executeQuery($query, $params);   
    }

    public function deleteEcomOper($id)
    {
        $query = <<<SQL
                DELETE
                  FROM INFORMATIVOECOMOPER
                 WHERE NRINFECOMMERCE = :NRINFECOMMERCE
                 AND NRORG = :NRORG
SQL;
        $params = array (
            'NRORG' => $this->getNrorg(),
            'NRINFECOMMERCE'    => $id,
        );
        $entityManager = InstanceProvider::getLibConnectionCustom()->updateVpdConnection($this->getNrorg());
        $entityManager = $entityManager ? $entityManager : InstanceProvider::getEntityManager();
        return $entityManager->getConnection()->executeQuery($query, $params);
    }

    public function updateNotice($item)
    {
        $query = <<<SQL
            UPDATE INFORMATIVOECOMMERCE 
            SET DSTITULOINF = :TITULO, 
                TXCONTEUDOINF = :CONTEUDO, 
                IDNOTIFICACAO = :TIPO, 
                DTINFORMARIVI = TRUNC(CAST(:DATAINICIO AS DATE)),
                DTEXPIRAINFO = TRUNC(CAST(:DATATERMINO AS DATE)), 
                DTULTATU = TRUNC(CAST(SYSDATE AS DATE)),
                IDATIVO = :ATIVO,
                NRORG = :NRORG,
                NRORGULTATU = :NRORGULTATU,
                CDOPERULTATU  = :CDOPERULTATU
            WHERE NRINFECOMMERCE = :ID
            AND NRORG = :NRORG
SQL;
        $params = array(
            'ID' => $item['id'],
            'TITULO' => $item['title'],
            'CONTEUDO' => $item['content'],
            'TIPO' => $item['type'],
            'DATAINICIO' => $item['dateBegin'],
            'DATATERMINO' => $item['dateEnd'],
            'ATIVO' => $item['active'],
            'NRORG' => $this->getNrorg(),
            'NRORGULTATU' => $this->getNrorg(),
            'CDOPERULTATU'  => $this->getCdoperador(),
        );
        $entityManager = InstanceProvider::getLibConnectionCustom()->updateVpdConnection($this->getNrorg());
        $entityManager = $entityManager ? $entityManager : InstanceProvider::getEntityManager();
        return $entityManager->getConnection()->executeQuery($query, $params);
    }

    public function getTimeOutnotice()
    {
        $query = <<<SQL
        SELECT ID,
               DSNOME,
               QTDIAS,
               IDATIVO
          FROM INFORMATIVOECOMSILE
          WHERE CDPUBLICOALVO = 'LOGIN'
          AND NRORG = :NRORG
SQL;
    $params = array(
        'NRORG' => $this->getNrorg(),
    );
        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $query, $params);
    }

    public function saveTimeOutNotice($connection, $id, $name, $qtDias, $cdoperador, $nrorg)
    {
        $query = <<<SQL
        INSERT INTO INFORMATIVOECOMSILE
               (ID, DSNOME, QTDIAS, IDATIVO, NRORG, DTINCLUSAO, NRORGINCLUSAO, CDOPERINCLUSAO, CDPUBLICOALVO)
        VALUES (:ID, :DSNOME, :QTDIAS, 'S', :NRORG, SYSDATE, :NRORGINCLUSAO, :CDOPERINCLUSAO, 'LOGIN')
SQL;
        $params = array(
            'ID' => $id,
            'DSNOME' => $name,
            'QTDIAS' => $qtDias,
            'NRORG' => $nrorg,
            'NRORGINCLUSAO' => $nrorg,
            'CDOPERINCLUSAO' => $cdoperador
        );

        $connection->executeStatement($query, $params);
    }

    public function updateTimeOutNotice($connection, $id, $name, $qtDias, $cdoperador, $nrorg)
    {
        $query = <<<SQL
        UPDATE INFORMATIVOECOMSILE
            SET DSNOME = :DSNOME,
                QTDIAS = :QTDIAS,
                IDATIVO = 'S',
                NRORG = :NRORG,
                DTULTATU = SYSDATE,
                NRORGULTATU = :NRORGULTATU,
                CDOPERULTATU = :CDOPERULTATU
        WHERE ID = :ID
SQL;
        $params = array(
            'ID' => $id,
            'DSNOME' => $name,
            'QTDIAS' => $qtDias,
            'NRORG' => $nrorg,
            'NRORGULTATU' => $nrorg,
            'CDOPERULTATU' => $cdoperador
        );

        $connection->executeStatement($query, $params);
    }

    public function deleteTimeOutNotice($connection, $id)
    {
        $query = <<<SQL
            DELETE FROM INFORMATIVOECOMSILE
            WHERE ID = :ID
            AND NRORG = :NRORG
SQL;
        $params = array(
            'ID' => $id,
            'NRORG' => $this->getNrorg()
        );

        $connection->executeQuery($query, $params);
    }

    public function validaRegistroIgual($connection, $id, $name)
    {
        $query = <<<SQL
        SELECT COUNT(*) AS QTD
        FROM INFORMATIVOECOMSILE
        WHERE DSNOME = :DSNOME
        AND ID != :ID
        AND CDPUBLICOALVO = 'LOGIN'
        AND NRORG = :NRORG
SQL;

        $params = array(
            'ID' => $id,
            'DSNOME' => $name,
            'NRORG' => $this->getNrorg()
        );

        return $connection->executeQuery($query, $params)->fetchAllAssociative();
    }

    public function getTimeOutnotices()
    {
        $query = <<<SQL
        SELECT ID,
               DSNOME,
               QTDIAS,
               IDATIVO
          FROM INFORMATIVOECOMSILE
          WHERE NRORG = :NRORG
SQL;
        $params = array(
            'NRORG' => $this->getNrorg()
        );
        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $query, $params);
    }

    public function getNoticesLogin($nrorg)
    {
        $query = <<<SQL
        SELECT DSTITULOINF, TXCONTEUDOINF, IDNOTIFICACAO, DTEXPIRAINFO, DTINFORMARIVI, IDATIVO,
               NRINFECOMMERCE, CDPUBLICOALVO, MARKISREAD
          FROM INFORMATIVOECOMMERCE
         WHERE CDPUBLICOALVO = 'L'
           AND TRUNC(SYSDATE) BETWEEN TRUNC(DTINFORMARIVI) AND TRUNC(DTEXPIRAINFO)
           AND NRORG = :NRORG
           AND IDATIVO = 'S'
SQL;

        $params = array(
            'NRORG' => $nrorg,
        );

        $entityManager = InstanceProvider::getLibConnectionCustom()->updateVpdConnection($nrorg);
        $entityManager = $entityManager ? $entityManager : InstanceProvider::getEntityManager();
        return $entityManager->getConnection()->fetchAll($query, $params);
    }

    public function getTimeOutNoticesLogin($nrorg)
    {
        $query = <<<SQL
        SELECT ID, DSNOME, QTDIAS, CDPUBLICOALVO, IDATIVO
        FROM INFORMATIVOECOMSILE
          WHERE CDPUBLICOALVO = 'LOGIN'
          AND NRORG = :NRORG
          AND IDATIVO = 'S'
SQL;

        $params = array(
            'NRORG' => $nrorg,
        );

        $entityManager = InstanceProvider::getLibConnectionCustom()->updateVpdConnection($nrorg);
        $entityManager = $entityManager ? $entityManager : InstanceProvider::getEntityManager();
        return $entityManager->getConnection()->fetchAll($query, $params);
    }

    public function getInforEcomOper($cdPublicoAlvo, $cdOperador)
    {
        $query =
            <<<SQL
                SELECT ID FROM INFORMATIVOECOMSILEOPER
                WHERE CDPUBLICOALVO = :CDPUBLICOALVO
                AND CDOPERADOR = :CDOPERADOR
                AND IDATIVO = 'S'
                AND NRORG = :NRORG

SQL;
        $params = array(
            'CDOPERADOR' => $cdOperador,
            'CDPUBLICOALVO' => $cdPublicoAlvo,
            'NRORG' => $this->getNrorg()
        );

        $result = InstanceProvider::getEntityManager()->getConnection()->fetchAll($query, $params);
        return $result;
    }

    public function saveUserNoticeSilence($connection, $id, $cdoperador, $cdPublicoAlvo, $dtInfoSile, $nrOrg)
    {
        $query = <<<SQL
        INSERT INTO INFORMATIVOECOMSILEOPER
               (ID, CDOPERADOR, CDPUBLICOALVO, DTINFOSILE, IDATIVO, NRORG, DTINCLUSAO, NRORGINCLUSAO, CDOPERINCLUSAO)
        VALUES (:ID, :CDOPERADOR, :CDPUBLICOALVO, :DTINFOSILE, 'S', :NRORG, SYSDATE, :NRORGINCLUSAO, :CDOPERINCLUSAO)
SQL;
        $params = array(
            'ID' => $id,
            'CDOPERADOR' => $cdoperador,
            'CDPUBLICOALVO' => $cdPublicoAlvo,
            'DTINFOSILE'    => $dtInfoSile,
            'NRORG' => $nrOrg,
            'NRORGINCLUSAO' => $nrOrg,
            'CDOPERINCLUSAO' => $cdoperador,
        );

        $connection->executeStatement($query, $params);
    }

    public function updateUserNoticeSilence($connection, $id, $dtInfoSile, $cdoperador, $nrorg)
    {
        $query = <<<SQL
        UPDATE INFORMATIVOECOMSILEOPER
            SET DTINFOSILE = :DTINFOSILE,
                NRORG = :NRORG,
                DTULTATU = SYSDATE,
                NRORGULTATU = :NRORGULTATU,
                CDOPERULTATU = :CDOPERULTATU
        WHERE ID = :ID
SQL;
        $params = array(
            'ID' => $id,
            'DTINFOSILE' => $dtInfoSile,
            'NRORG' => $nrorg,
            'NRORGULTATU' => $nrorg,
            'CDOPERULTATU' => $cdoperador
        );

        $connection->executeStatement($query, $params);
    }

    public function getDtInforEcomOper($cdOperador, $nrorg, $cdPublicoAlvo = 'LOGIN')
    {
        $query =
            <<<SQL
                SELECT TO_CHAR(DTINFOSILE, 'DD/MM/YYYY') || ' 23:59:59' AS DTINFOSILE 
                  FROM INFORMATIVOECOMSILEOPER
                 WHERE CDPUBLICOALVO = :CDPUBLICOALVO
                   AND CDOPERADOR = :CDOPERADOR
                   AND NRORG = :NRORG
SQL;
        $params = array(
            'CDOPERADOR' => $cdOperador,
            'NRORG' => $nrorg,
            'CDPUBLICOALVO' => $cdPublicoAlvo,
        );

        $entityManager = InstanceProvider::getLibConnectionCustom()->updateVpdConnection($nrorg);
        $entityManager = $entityManager ? $entityManager : InstanceProvider::getEntityManager();
        return $entityManager->getConnection()->fetchAll($query, $params);
    }

    public function getNoticesMarkIsRead($cdOperador, $nrorg)
    {
        $query =
            <<<SQL
                SELECT NRINFECOMMERCE 
                  FROM INFORMATIVOECOMOPER
                 WHERE CDOPERADOR = :CDOPERADOR
                   AND IDATIVO = 'S'
                   AND NRORG = :NRORG

SQL;
        $params = array(
            'CDOPERADOR' => $cdOperador,
            'NRORG' => $nrorg
        );

        $entityManager = InstanceProvider::getLibConnectionCustom()->updateVpdConnection($nrorg);
        $entityManager = $entityManager ? $entityManager : InstanceProvider::getEntityManager();
        return $entityManager->getConnection()->fetchAll($query, $params);
    }

    public function readNotification($nrinfecomoper, $notificationId, $cdoperador, $nrOrg) {
        $query = <<<SQL
        INSERT INTO INFORMATIVOECOMOPER
               (NRINFECOMOPER, NRINFECOMMERCE, CDOPERADOR, NRORG, IDATIVO, DTINCLUSAO, NRORGINCLUSAO, CDOPERINCLUSAO)
        VALUES (:NRINFECOMOPER, :NRINFECOMMERCE, :CDOPERADOR, :NRORG, 'S', SYSDATE, :NRORGINCLUSAO, :CDOPERINCLUSAO)
SQL;
        $params = array(
            "NRINFECOMOPER" => $nrinfecomoper,
            "NRINFECOMMERCE" => $notificationId,
            "CDOPERADOR" => $cdoperador,
            "NRORG"     => $nrOrg,
            "NRORGINCLUSAO" => $nrOrg,
            "CDOPERINCLUSAO"    => $cdoperador
        );
        $entityManager = InstanceProvider::getLibConnectionCustom()->updateVpdConnection($this->getNrorg());
        $entityManager = $entityManager ? $entityManager : InstanceProvider::getEntityManager();
        return $entityManager->getConnection()->executeQuery($query, $params);
    }
}
