import { Utilities } from '@/util/Utilities';
import { Carousel, DialogService, LoadingService, Select, Text } from '@zeedhi/common';
import { Http, Metadata } from '@zeedhi/core';

export class ViewNoticeForOperatorController {
    public itemUrl: string = '';

    public getTextAnuncio: string = '';

    private notices: any[] = [];

    private currentSlideIndex: number = 0;

    public getTextNoticeTeknisa: string = '';

    public visibleTextNoticeTeknisa: boolean = false;

    public visibilidadeInternalNotices: boolean = false;

    public visibilidadeTeknisaNotices: boolean = false;

    public markIsRead: any[] = [];

    get visibleCloseNoticeTeknisa() {
        const isNext = localStorage.getItem('IS_NEXT');
        if (!isNext || isNext === 'false') {
            return false;
        }
        return true;
    }

    public async setIFrameDocSource({ component }: any) {
        if (this.itemUrl) {
            const doomIframes: HTMLIFrameElement[] = Array.from(document.querySelectorAll(`#${component.name}`));
            doomIframes.forEach((element: HTMLIFrameElement) => {
                element.src = `${this.itemUrl}#zoom=76`;
            });
        }
    }

    public titleNotice({ component }: any) {
        this.currentSlideIndex = component.currentSlide;
        this.updateTitle();
    }

    private updateTitle() {
        const currentNotice = this.notices[this.currentSlideIndex];
        if (currentNotice) {
            if (currentNotice.CDPUBLICOALVO === 'T' || currentNotice.CDPUBLICOALVO === 'A' || currentNotice.CDPUBLICOALVO === 'C') {
                this.visibleTextNoticeTeknisa = true;
                this.getTextNoticeTeknisa = 'Comunicado Teknisa';
            } else {
                this.visibleTextNoticeTeknisa = false;
                this.getTextNoticeTeknisa = '';
            }
            this.getTextAnuncio = currentNotice.DSTITULOINF;
            const title = Metadata.getInstance<Text>('viewNotice-title');
            title.text = this.getTextAnuncio;
        }
    }

    public onMountedTimeOutNotice() {
        const timeOutnoticesJson: any = localStorage.getItem('TIMEOUTNOTICES');
        const timeOutnotices = JSON.parse(timeOutnoticesJson);
        const internalTimeOutArray: any = [];
        const teknisaTimeOutArray: any = [];
        timeOutnotices.forEach((element: any) => {
            if (element.CDPUBLICOALVO === 'LOGIN') {
                internalTimeOutArray.push(element);
            }
            if (element.CDPUBLICOALVO === 'OM') {
                teknisaTimeOutArray.push(element);
            }
        });
        if (internalTimeOutArray.length > 0) {
            this.visibilidadeInternalNotices = true;
        }
        if (teknisaTimeOutArray.length > 0) {
            this.visibilidadeTeknisaNotices = true;
        }
        Metadata.getInstance<Select>('noticesForInternalViewNotice').datasource.data = internalTimeOutArray;
        Metadata.getInstance<Select>('noticesForTeknisaViewNotice').datasource.data = teknisaTimeOutArray;
    }

    public onMountedNoticeView() {
        LoadingService.show();
        Http.post('/getSessionNotifications', {
            LOGIN_TOKEN: localStorage.getItem('LOGIN_TOKEN'),
        }).then((noticesInfo: any) => {
            localStorage.setItem('TIMEOUTNOTICES', JSON.stringify(noticesInfo.data.TIMEOUTNOTICES));
            const notices = noticesInfo.data.NOTICES;
            this.notices = notices;
            const carouselChildren = notices.map((notice: any, index: any) => {
                const name = `slide-${index}`;
                switch (notice.IDNOTIFICACAO) {
                    case 'I':
                        this.getTextAnuncio = notice.DSTITULOINF;
                        return {
                            id: notice.NRINFECOMMERCE,
                            name,
                            component: 'ZdCard',
                            height: '100%',
                            width: '100%',
                            cssStyle: {
                                display: 'flex',
                                justifyContent: 'center',
                                alignItems: 'center',
                            },
                            children: [
                                {
                                    name: `${name}-image`,
                                    component: 'ZdRow',
                                    justify: 'center',
                                    align: 'center',
                                    children: [
                                        {
                                            name: 'preView_content',
                                            component: 'ZdImage',
                                            src: notice.TXCONTEUDOINF,
                                            cssStyle: {
                                                maxWidth: '100%',
                                                maxHeight: '100%',
                                                objectFit: 'contain',
                                            },
                                        },
                                    ],
                                },
                            ],
                        };
                    case 'H':
                        this.getTextAnuncio = notice.DSTITULOINF;
                        return {
                            id: notice.NRINFECOMMERCE,
                            name,
                            component: 'ZdCard',
                            height: '100%',
                            width: '100%',
                            children: [
                                {
                                    name: `${name}-text`,
                                    component: 'ZdText',
                                    text: notice.TXCONTEUDOINF,
                                },
                            ],
                        };
                    case 'P':
                        this.getTextAnuncio = notice.DSTITULOINF;
                        this.itemUrl = notice.TXCONTEUDOINF;
                        return {
                            id: notice.NRINFECOMMERCE,
                            name,
                            component: 'ZdCard',
                            height: '100%',
                            width: '100%',
                            children: [
                                {
                                    name: `${name}-iframe`,
                                    id: `${name}-iframe`,
                                    component: 'ZdTag',
                                    tag: 'iframe',
                                    cssStyle: 'width: 100%; height: calc(-7px + 100%)',
                                    events: {
                                        onMounted: '{{ViewNoticeForOperatorController.setIFrameDocSource}}',
                                    },
                                },
                            ],
                        };
                    default:
                        return null;
                }
            }).filter((item: any): item is NonNullable<typeof item> => item !== null);
            const componentCarousel = Metadata.getInstance<Carousel>('viewNoticeCarousel');
            if (componentCarousel.children) {
                componentCarousel.children.push(...carouselChildren);
            } else {
                componentCarousel.children = carouselChildren;
            }
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
        });
    }

    public async changeHintSilencers({ component }: any) {
        if (component.selectValue) {
            let message = '';
            const daysToSilence = parseInt(component.selectValue.QTDIAS, 10);
            if (daysToSilence === 0) {
                message = 'Silenciado para sempre';
            } else {
                const currentDate = new Date();
                const targetDate = new Date(currentDate);
                targetDate.setDate(currentDate.getDate() + daysToSilence);
                const day = targetDate.getDate().toString().padStart(2, '0');
                const month = (targetDate.getMonth() + 1).toString().padStart(2, '0');
                const year = targetDate.getFullYear();
                const formattedDate = `${day}/${month}/${year}`;
                message = `Silenciado até ${formattedDate}`;
            }
            component.hint = message;
        }
    }

    public async closeModalView() {
        window.parent.postMessage({
            type: 'pageViewNoticeCard',
        }, <string>localStorage.getItem('PRODUCT_URL'));
    }

    public async readAsMark() {
        const componentCarousel = Metadata.getInstance<Carousel>('viewNoticeCarousel');
        let currentSlides = componentCarousel.currentSlide;
        if (this.markIsRead.includes(currentSlides)) {
            DialogService.show({
                name: 'markIsRead-dialog',
                type: 'warning',
                title: 'Atenção',
                text: 'Esse comunicado já foi marcado como lido.',
            });
        } else {
            LoadingService.show();
            Http.post('/readNotification', {
                NRINFECOMMERCE: componentCarousel.children[currentSlides].id,
            }).then(() => {
                this.markIsRead.push(currentSlides);
                currentSlides += 1;
                componentCarousel.currentSlide = currentSlides;
                Utilities.showSuccessMessageNotice();
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                LoadingService.hide();
            });
        }
    }

    public get visibilidadeButton() {
        const timeOutnoticesJson: any = localStorage.getItem('TIMEOUTNOTICES');
        const timeOutnotices = JSON.parse(timeOutnoticesJson);
        if (timeOutnotices.length === 0) {
            return false;
        }
        return true;
    }

    public saveUserNoticeSilence() {
        const timeOutNotices = [];
        const noticesForInternalViewNotice = Metadata.getInstance<Select>('noticesForInternalViewNotice');
        const noticesForTeknisaViewNotice = Metadata.getInstance<Select>('noticesForTeknisaViewNotice');
        if (noticesForInternalViewNotice.selectValue !== null) {
            timeOutNotices.push(noticesForInternalViewNotice.selectValue);
        }
        if (noticesForTeknisaViewNotice.selectValue !== null) {
            timeOutNotices.push(noticesForTeknisaViewNotice.selectValue);
        }
        LoadingService.show();
        Http.post('/saveUserNoticeSilence', {
            TIME_OUT_NOTICE: timeOutNotices,
        }).then(() => {
            Utilities.showSuccessMessage();
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
        });
    }
}
